@@ -4,9 +4,13 @@ Collection of functions for accelerator physics for PhD projects  - local test v
 Author: Elias Waagaard, elias.walter.waagaard@cern.ch
 """
 import numpy as np
-import matplotlib.pyplot as plt
 import pandas as pd
+
+import matplotlib.pyplot as plt
 import matplotlib as mpl
+import matplotlib.patches
+import matplotlib.collections
+
 from scipy.fft import fft, fftfreq
 import NAFFlib # Numerical Analysis of the Fundamental Frequencies 
 
@@ -307,6 +311,7 @@ class resonance_lines(object):
         else:
             fig = plt.figure()
         ax = fig.add_subplot(1, 1, 1)  # create an axes object in the figure
+        fig.suptitle('Tune footprint and resonances up to order {}'.format(self.orders[-1]), fontsize=18)
         
         Qx_min = self.Qx_min
         Qx_max = self.Qx_max
@@ -557,4 +562,79 @@ class madx_tools:
             )  #previously step="pre"
             ax.plot(s, aper, "k", label='_nolegend_')  #previously drawstyle="steps"
             ax.plot(s, aper_flipped, "k", label='_nolegend_')   #previously drawstyle="steps"
- 
\ No newline at end of file
+ 
+
+class footprint: 
+    """
+    Class to convert real to normalized phase space coordinates, and to draw footprints
+    
+    Example following the X-suite space charge example on GitHub:
+    https://github.com/xsuite/xtrack/blob/main/examples/spacecharge/001_spacecharge_footprint.py
+    """    
+
+    def initial_xy_polar(r_min, r_max, r_N, theta_min, theta_max, theta_N):
+        """ 
+        Return grid of x and y coordinates from range of polar coordinates
+        """
+        return np.array([[(r*np.cos(theta),r*np.sin(theta)) for r in np.linspace(r_min,r_max,r_N)] for theta in np.linspace(theta_min,theta_max,theta_N)])
+
+    def initial_xy_cartesian(x_min, x_max, x_N, y_min, y_max, y_N):
+        """
+        Return x-y grid of x and y coordinates from range of cartesian coordinates 
+        """
+        return np.array([[(x,y) for x in np.linspace(x_min,x_max,x_N)] for y in np.linspace(y_min,y_max,y_N)])
+
+
+    def draw_footprint(A, axis_object=None, figure_object=None, axis=0, linewidth=4):
+        """
+        Input A should be a 3-D numpy array with shape (Nx,Ny,2)
+        representing a 2-D array of (x,y) points. This function
+        will draw lines between adjacent points in the 2-D array.
+        """
+        if len(A.shape) != 3:
+            print('ERROR: Invalid input matrix')
+            return None
+        if A.shape[2] != 2:
+            print('ERROR: Points are not defined in 2D space')
+            return None
+
+        sx = A.shape[0]-1
+        sy = A.shape[1]-1
+
+        p1 = A[:-1,:-1,:].reshape(sx*sy,2)[:,:]
+        p2 = A[1:,:-1,:].reshape(sx*sy,2)[:]
+        p3 = A[1:,1:,:].reshape(sx*sy,2)[:]
+        p4 = A[:-1,1:,:].reshape(sx*sy,2)[:]
+
+        #Stack endpoints to form polygons
+        Polygons = np.stack((p1,p2,p3,p4))
+        #transpose polygons
+        Polygons = np.transpose(Polygons,(1,0,2))
+        patches = list(map(matplotlib.patches.Polygon,Polygons))
+
+        #assign colors
+        patch_colors = [(0,0,0) for a in Polygons]
+        patch_colors[(sx-1)*sy:] = [(0,1,0)]*sy
+        patch_colors[(sy-1)::sy] = [(0,0,1)]*sx
+
+        p_collection = matplotlib.collections.PatchCollection(patches,facecolors=[],linewidth=linewidth,edgecolor=patch_colors)
+
+        if axis_object is None:
+            if figure_object:
+                fig = figure_object
+            else:
+                fig = plt.figure()
+            if len(fig.axes) == 0:
+                plt.subplot(1,1,1)
+            if axis >= len(fig.axes) or axis < 0:
+                i = 0
+            else:
+                i = axis
+            ax = fig.axes[i]
+        else:
+            ax = axis_object
+            fig = None
+
+        ax.add_collection(p_collection)
+
+        return fig
\ No newline at end of file
